#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Inspire by a discussion found here: http://gimpchat.com/viewtopic.php?f=8&t=19549&start=10&hilit=sugar
# author: Pocholo
# date: 9/10/21

# Comments directed to http://gimpchat.com

# Installation:
# This script should be placed in the user plugin folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.


from gimpfu import *
import math
import random
import gettext
import os, sys,time 
gettext.install("gimp20", gimp.locale_directory, unicode=True)

def pm_sugar_or_salt_text_v2(img, draw, brightness):
   
    if draw== False:
        pdb.gimp_message("No,no,no... You need to create a Text")
    
    #Start undo
    pdb.gimp_image_undo_group_start(img)
    
    #Initiates the temporary state
    pdb.gimp_context_push()
    pdb.gimp_context_set_defaults()

    #Make active the image 
    draw = pdb.gimp_image_get_active_drawable(img)       
    pdb.gimp_image_set_active_layer(img, draw)
    pdb.gimp_item_set_name(draw, "Background")
    
    #Make active the text 
    textLayer = pdb.gimp_image_get_active_drawable(img)       
    pdb.gimp_image_set_active_layer(img, textLayer)
    pdb.gimp_item_set_name(textLayer, "Text")
    
    t_width = pdb.gimp_drawable_width(textLayer)
    t_height = pdb.gimp_drawable_height(textLayer)
    
    #Create the effect 
    non_empty = pdb.gimp_edit_copy(textLayer)
    textCopy = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Effect", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, textCopy, 0)
    pdb.gimp_drawable_fill(textCopy, FILL_TRANSPARENT)
    floating_sel = pdb.gimp_edit_paste(textCopy, FALSE)
    pdb.gimp_floating_sel_anchor(floating_sel)
    textCopy = pdb.gimp_image_get_layer_by_name(img, "Effect")
    pdb.gimp_image_select_item(img, 2, textCopy)
    pdb.gimp_edit_bucket_fill(textCopy,  
                        BUCKET_FILL_FG,  #fill_mode
                     LAYER_MODE_NORMAL,  #paint_mode
                                   100,  #opacity
                                   255,  #threshold
                                 FALSE,  #sample_merged
                                     0,  #x
                                     0)  #y
    pdb.plug_in_hsv_noise(img, textCopy, 4, 0, 0, 255)    
    pdb.gimp_selection_none(img)
    pdb.plug_in_spread(img, textCopy, 50, 50)
    pdb.plug_in_spread(img, textLayer, 30, 30)
      
    #Create sugar or salt random grains
    grainLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Random Grains" , 100, LAYER_MODE_SCREEN)
    pdb.gimp_image_add_layer(img, grainLayer, 0)
    pdb.gimp_context_set_foreground("Black")
    pdb.gimp_drawable_fill(grainLayer, FILL_FOREGROUND)
    pdb.plug_in_hsv_noise(img, grainLayer, 4, 0, 0, 255)
    pdb.gimp_brightness_contrast(grainLayer, -100, 127)
    pdb.plug_in_gauss(img, grainLayer, 0.5, 0.5, 1)
    pdb.plug_in_normalize(img, grainLayer)
    
    #Make sugar brighter
    brightLayer = pdb.gimp_layer_new_from_drawable(textCopy, img)
    pdb.gimp_image_insert_layer(img, brightLayer, None, 1)
    pdb.gimp_layer_set_mode(brightLayer, LAYER_MODE_DIVIDE)
    pdb.gimp_layer_set_opacity(brightLayer, brightness)
    pdb.gimp_item_set_name(brightLayer, "Brightness")
    
    #Set the bottom layer active
    backLayer = pdb.gimp_image_get_layer_by_name(img, "Background")
    pdb.gimp_image_set_active_layer(img, backLayer)
    
    pdb.gimp_displays_flush()
    
    #Set gimp to default
    pdb.gimp_context_pop()
    
    #End undo
    pdb.gimp_image_undo_group_end(img)
    
register(
    "pm_sugar_or_salt_text_v2",
    "Creates a Sugar or Salt text effect",
    "Creates a Sugar or Salt text effect",
    "Pocholo",
    "Pocholo",
    "2022",
    "Sugar or Salt Text Effect_v2",
    "",
    [
    (PF_IMAGE,   "img", "Input image", 0),
    (PF_DRAWABLE,   "drawable", "Input drawable", None),
    (PF_SPINNER, "brightness", "Brightness:", 100, (0.1, 100, 0.1)),
    ],
    [],
    pm_sugar_or_salt_text_v2, menu="<Image>/Pocholo's-Plugins/1.Text Effect/",
    domain=("gimp20", gimp.locale_directory))
main()	